////////////////////////////////////////////////////////////////////// 
// merlin/KAC.h 
// (c) 2000-2001 Goncalo Abecasis
// 
// This file is distributed as part of the MERLIN source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile MERLIN.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Monday December 17, 2001
// 
 
#ifndef __KAC_H__
#define __KAC_H__

#include "Pedigree.h"
#include "MathMatrix.h"

class KAC : public VectorFunc
   {
   public:
      int affectionCount, familyCount, positionCount;
      Vector * min;         // min[affection][family]
      Vector * max;         // max[affection][family]
      Matrix * scores;      // scores[affection][family][position]
      StringArray & labels; // labels for analysis positions
      StringArray pheno;    // labels for phenotypes

      KAC(StringArray & labels);
      ~KAC();

      void LabelPhenotypes(Pedigree & ped,
                           bool all, bool pairs, bool qtl, bool dev);
      void Prepare(int affections, int families);
      void Set(int affection, int family, double lo, double hi, Vector & z);
      void Analyse();

      void SkipFamily(int serial);

   private:
      double Evaluate(Vector & parameters);

      int    currentAffection, currentPosition;
      double minimum, range;

      double MakeDelta(double value);
      void   OutputScores(const char * pos, double z, double d, double chisq);
   };

#endif
 
